// udStFiLrXML
// J. J. Weimer
// History: $Revision: 471 $ on $Date: 2008-01-16 14:38:04 -0500 (Wed, 16 Jan 2008) $
//
// Procedure files to parse an XML file and store it
//
// The XML file must have an XSL file containing definitions for
// XSL:TEMPLATE MATCH="//string": the name of each item for string wave storage
// VALUE-OF SELECT="string": the name of the value a given item
//
// The XSL style values are stored in root:Packages:XML2IgorPro:filenameXSL
// ename: string wave of SELECT values
// sname: string wave of STORE values
//
// Access to Package
//
// -> udFileLoaderXML(udFL)
//		STRUCTURE udFLStandardStructure &udFL
//
// see the structure definition and User's Manual

#pragma version=3.00
#pragma rtGlobals=1
#pragma IgorVersion=6.0

#include ":udStFiLrXML.lcl"

#include "udFLStandardStructure"
#include "udFLStandardFunctions"

// **********************************************
// Globals and Localizations

StrConstant XMLcr = "\r"

Static Structure xml2igorproglobals
	string version
	variable xslversionsupported
EndStructure

Static Function Initxml2igorproglobals(xip)
	STRUCT xml2igorproglobals &xip

	xip.version="3.00"
	xip.xslversionsupported=1.04
End

Static Function udFLXMLInititalize(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFLZeroStructure(udFL)
	udFLXMLQuery(udFL)
end

Static Function udFLXMLQuery(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFL.mimetype = "TEXT"
	udFL.extensions = ".xsl;.xml;"
	udFL.procModes = "Auto;Auto XML;Manual;"
	udFL.itemsDim[p] = p*0
	udFL.setNames=1
	
	switch(udFL.itemsType)
		case 1:
			udFL.itemsDim[0]=3		// variable or string
			break
		case 2:
			udFL.itemsDim[0]=3		// keyword = tag or keyword : tag
			break
		case 4:
			udFL.itemsDim[0] = 0		// text or numeric waves set by XSL?
			break
		default:
			udFL.itemsType = (2^0 + 2^1 + 2^2)		// mixture of returns
			udFL.itemsDim[0] = 0
	endswitch
	
	return 0
	
end

// **********************************************
// File Loader

Function udStFiLrXML(udFL)
	STRUCT udFLStandardStructure &udFL

	variable refNum
	string pathStr, fileStr
	
	if (udFL.eventCode == -1)
		SetDataFolder root:
		KillDataFolder/Z root:Packages:XML2igorpro
		return 0
	endif
	
	switch(udFL.userCtrl)
		case -1:		// initialize		
			if (!DataFolderExists("root:Packages:xml2igorpro"))
				initParseXML()
			endif
			udFLXMLInititalize(udFL)		
			return 0
		case 0:		// query
			udFLXMLQuery(udFL)
			return 0
		case 1:
			if (strlen(udFL.fileList)==0)
				udFLSetError(udFL,XMLalert0a,-1)
				return -1
			endif
			if (udFL.reportCtrl == 1)
				print "... confirming list of file names"
			endif
			confirmXMLFileList(udFL)
			if (udFL.reportCtrl == 1)
				print "... assuring file paths"
			endif
			if (udFLAssureFilePaths(udFL)<0)
				return -1
			endif
			break
		case 2:		// find XSL, use the name to find XML
			if (strlen(udFL.fileList) == 0)
				Open/D/R/M=xmldistr00/T=".xsl" refNum
				if (strlen(S_filename)!=0)
					udFL.filelist = S_filename + ";"
					udFL.fileList += ParseFilePath(1, S_filename, ":", 1, 0) + ParseFilePath(3, S_filename, ":", 0, 0) + ".xml;"
				else
					udFLSetError(udFL,XMLalert06,-1)
					return -1
				endif
			endif
			if (udFL.reportCtrl == 1)
				print "... assuring file paths"
			endif
			if (udFLAssureFilePaths(udFL)<0)
				return -1
			endif
			if (udFL.reportCtrl == 1)
				print "... searching for XML"
			endif
			pathStr = udFL.pathStr
			NewPath/O/Q tmpXMLDir, pathStr
			fileStr = StringFromList(1,udFL.fileList)
			Open/Z=1/R/P=tmpXMLDir refNum as fileStr
			if (V_flag!=0)
				udFLSetError(udFL,XMLalert05,-1)
				KillPath/Z tmpXMLDir
				return -1
			endif
			break
		case 3:		// find both XSL and XML
			Open/D/R/M=xmldistr00/T=".xsl" refNum
			if (strlen(S_filename)!=0)
				udFL.fileList = S_filename + ";"
			else
				udFLSetError(udFL,XMLalert05,-1)
				return -1
			endif
			pathStr = ParseFilePath(1,StringFromList(0,udFL.fileList),":",1,0)
			NewPath/O/Q tmpXMLDir, pathStr
			Open/D/R/M=xmldistr01/T=".xml"/P=tmpXMLDir refNum
			if (strlen(S_filename)!=0)
				udFL.fileList += S_filename + ";"
				udFL.pathStr = ""
			else
				udFLSetError(udFL,XMLalert05,-1)
				KillPath/Z tmpXMLDir
				return -1
			endif
			if (udFL.reportCtrl == 1)
				print "... assuring file paths"
			endif
			if (udFLAssureFilePaths(udFL)<0)
				return -1
			endif
			break
	endswitch
	if (udFL.reportCtrl == 1)
		print "... checking XSL"
	endif
	if (checkXSLFile(udFL)<0)
		KillPath/Z tmpXMLDir
		return -1
	endif
	if (udFL.reportCtrl == 1)
		print "... transforming XML"
	endif
	if (transformXMLFile(udFL)<0)
		KillPath/Z tmpXMLDir
		return -1
	endif

	if (udFL.reportCtrl == 1)
		print "... parsing XML"
	endif
	if (parseXMLFile(udFL)<0)
		KillPath/Z tmpXMLDir
		return -1
	endif

	if (udFL.reportCtrl == 1)
		print "... final udFL", udFL
	endif
	if (udFL.reportCtrl >= 0)
		print "... loaded data ", udFL.itemsList
	endif
	KillPath/Z tmpXMLDir
	return 0
end

// **********************************************
// ENTRY POINTS TO FUNCTIONS ARE ABOVE THIS LINE
// Functions BELOW this point are SPECIFIC to this PACKAGE
// If you want to use them, change their names!!!!

// confirmXMLFileList(udFL)
// makes sure first file is .xsl and others are .xml during auto mode

Static Function confirmXMLFileList(udFL)
	STRUCT udFLStandardStructure &udFL

	string flist = udFL.fileList, tmpi, tmpo
	
	tmpi = ListMatch(flist,"*.xsl")
	tmpo = RemoveFromList(tmpi,flist)
	if (strlen(ListMatch(tmpo,"*.xsl"))!=0)
		udFLSetError(udFL,XMLalert09,-1)
		return -1
	endif
	
	udFL.fileList = tmpi + tmpo
	
	return 0	
end

// initParseXML()
// create package folders
// return: 0-already existed; 1-newly created

Static Function initParseXML()

//	string cdf=GetDataFolder(1)
//	variable rc = 0
//
//	NewDataFolder/O/S root:Packages
//	if (!DataFolderExists("root:Packages:XML2IgorPro"))
//		NewDataFolder/S XML2IgorPro
//		string/G currXMLData, currXSLData
//		rc = 1
//	endif
//	
//	SetDataFolder $cdf
//	
//	return rc
	return 0
end

// **********************************************
// readXSLFile(udFL)
// check the XSL style sheet for igorversion information
// return: -1-unsuccessful; 0-successful

Static Function checkXSLFile(udFL)
	STRUCT udFLStandardStructure &udFL
	
	variable refNum, ic, pos
	variable xslversionread
	string fname, pathStr, theLine
	string theStr = "<?igorpro version="

	fname = StringFromList(0,udFL.fileList)
	if (strlen(udFL.pathStr)!=0)
		pathStr = udFL.pathStr	
		NewPath/O/Q tmpXMLDir, pathStr
		Open/Z/R/P=tmpXMLDir refNum as fname
	else
		Open/Z/R refNum as fname
	endif
	if (V_flag!=0)
		udFLSetError(udFL,XMLalert06b,-1)
		KillPath/Z tmpXMLDir
		return -1
	endif
	KillPath/Z tmpXMLDir

	STRUCT xml2igorproglobals xip
	Initxml2igorproglobals(xip)
	
	for (ic=0;ic<5;ic+=1)
		FReadLine refNum, theLine
		pos = strsearch(theLine,theStr,0,2)
		if (pos>=0)
			break
		endif
	endfor
	if (ic>4)
		udFLSetError(udFL,XMLalert00,-1)
		return -1
	endif
	
	pos=strsearch(theLine,"=",pos)
	xslversionread = str2num(theLine[pos+2,pos+5])
	if (xslversionread<xip.xslversionsupported)
		udFLSetError(udFL,XMLalert01a,-1)
		return -1
	endif
	if (xslversionread>xip.xslversionsupported)
		udFLSetError(udFL,XMLalert01b,-1)
		return -1
	endif
	
	udFL.itemsCount = 0
	return 0

end

// **********************************************
// transformXMLFile(udFL)
// transform the XML file into a notebook
// return: 0-unsuccessful; 1-successful

Function transformXMLFile(udFL)
	STRUCT udFLStandardStructure &udFL

	variable refNum
	string XSLfname,XMLfname, drStr="", XMLoutput, XMLoutputfile
	
	string theCmd = "", theUCmd =""
	
	if (strlen(udFL.pathStr)!=0)
		if (cmpstr(IgorInfo(2),"Macintosh")==0)
			XSLfname = "Volumes:" + udFL.pathStr + StringFromList(0,udFL.fileList)
			XMLfname = "Volumes:" + udFL.pathStr +  StringFromList(1,udFL.fileList)
		else
			drStr = udFL.pathStr[0,1] + "\\"
			XSLfname = udFL.pathStr[2,strlen(udFL.pathStr)-1] + StringFromList(0,udFL.fileList)
			XMLfname = udFL.pathStr[2,strlen(udFL.pathStr)-1] +  StringFromList(1,udFL.fileList)
			XMLoutput = "TMP_" + XMLfname[0,strlen(XMLFname)-5] + ".txt"
		endif
	else
		if (cmpstr(IgorInfo(2),"Macintosh")==0)
			XSLfname = "Volumes:" + StringFromList(0,udFL.fileList)
			XMLfname = "Volumes:" + StringFromList(1,udFL.fileList)	
		else
			drStr = udFL.fileList[0,1] + "\\"
			XMLoutput = StringFromList(0,udFL.fileList)
			XSLfname = XMLoutput[2,strlen(XMLoutput)-1]
			XMLoutput = StringFromList(1,udFL.fileList)
			XMLfname = XMLoutput[2,strlen(XMLoutput)-1]
		endif
	endif
	if (strlen(WinLIst("ParseXMLNB",";","WIN:16"))!=0)
		KillWindow ParseXMLNB
	endif
	
	if (cmpstr(IgorInfo(2),"Macintosh")==0)
		XSLfname = ReplaceString(":",XSLfname,"/")
		XMLfname = ReplaceString(":",XMLfname,"/")
		
		sprintf theUCmd, "xsltproc \\\"%s\\\" \\\"%s\\\"", XSLfname, XMLfname
		sprintf theCmd, "do shell script \"%s\"", theUcmd
//		print theCmd
		ExecuteScriptText/Z theCmd
		
		if (V_flag)
			udFLSetError(udFL,XMLalert04,-1)
			return -1
		endif
		
		if (strlen(S_value)==0)
			udFLSetError(udFL,XMLalert06c,-1)
			return -1
		endif
	
		if (udFL.reportCtrl != 1)
			NewNotebook/V=0/F=0/k=1/N=ParseXMLNB
		else
			NewNotebook/F=0/k=1/N=ParseXMLNB
		endif
		Notebook ParseXMLNB text=ReplaceString("><",S_value,">\r<")
	else
		XSLfname = drStr + ReplaceString(":",XSLfname,"\\")
		XMLfname = drStr + ReplaceString(":",XMLfname,"\\")
		XMLoutputfile = drStr + XMLoutput
		XMLoutput = drStr + ReplaceString(":",XMLoutput,"\\")
		sprintf theUCmd, "cmd.exe /q/c AltovaXML -xslt2 \"%s\" -in \"%s\" -out  \"%s\"", XSLfname, XMLfname, XMLoutput
		ExecuteScriptText/Z theUCmd
		if (V_flag)
			udFLSetError(udFL,XMLalert04,-1)
			return -1
		endif
		if (udFL.reportCtrl !=1)
			OpenNotebook/R/N=ParseXMLNB/k=1/V=0 XMLoutputfile
		else
			OpenNotebook/R/N=ParseXMLNB/k=1 XMLoutputfile
		endif
		sprintf theUCmd, "cmd.exe /c del \"%s\"",XMLoutput
//		print theUCmd
		ExecuteScriptText/Z theUCmd
	endif
	return 0

end

// **********************************************
// parseXMLFile(udFL)
// 

Static Function parseXMLFile(udFL)
	STRUCT udFLStandardStructure &udFL
	
//	SVAR currXMLData = root:Packages:XML2IgorPro:currXMLData
	
	// initialize
	
	udFL.itemsList = ""
	udFL.itemsCount = 0
	udFL.itemsDim[0] = 0
	
	// set data folder
	
	string cdf = GetDataFolder(1)
//	currXMLData = udFL.toFolder
	udFLSetDataFolder(udFL)
	
	// parse each line of notebook starting at top
	
	string nb = "ParseXMLNB", theHTML = "", theID = "", theDID = "", theScontent = ""
	string xnb = "ExecuteXMLCodeNB", tmpS, kwLStr = "=", theMID = ""
	variable iHTMLOK = 0, oHTMLOK = 0, isNumeric = 0
	variable tmpN, ic, mxic, stPr, enPr, enPo
	
	if (strlen(WinLIst(xnb,";","WIN:16"))!=0)
		KillWindow $xnb
	endif
	
	Notebook $nb selection={startOfFile, startOfFile}
	Notebook $nb selection={startOfParagraph, EndofParagraph}
	
	do
		GetSelection notebook, $nb, 3
		if (strlen(S_selection)!=0)

		// start of outer processing for storage options
			
		theHTML = getHTMLInfo(S_selection,1)
		if (strlen(theHTML)!=0)
			strswitch(theHTML)
			
				// internal processing of DIV
				
				case "DIV":
					theDID = getHTMLInfo(S_selection,3)
					if (strlen(theDID)==0)
						udFLSetError(udFL,XMLalert02,-1)
						return -1
					endif
					iHTMLOK=0
					do	
						Notebook $nb selection={startOfNextParagraph, EndOfNextParagraph}
						GetSelection notebook, $nb, 3
						theHTML = getHTMLInfo(S_selection,1)
						if (strlen(theHTML)!=0)
							strswitch(theHTML)
								case "p":
									theID = getHTMLInfo(S_selection,3)
									if (strlen(theID)==0)
										udFLSetError(udFL,XMLalert02,-1)
										return -1
									endif
									theScontent = getHTMLInfo(S_selection,0)
									if (cmpstr(theDID,"string")==0)
										theSContent = parseXMLStr(theSContent,0)
										string/G $theID = theSContent
										udFL.itemsDim[0] = 2
									else
										sscanf theScontent, "%g", tmpN
										variable/G $theID = tmpN
										udFL.itemsDim[0] = 1
									endif
									udFL.itemsType = 2^0
									udFL.itemsCount += 1
									udFL.itemsList += theID + ";"
									break
								case "/div":
									iHTMLOK = 1
									break
								default:
									break
							endswitch
						endif
					while(!iHTMLOK)
					break
					
				// internal processing of lists
				
				case "ul":
					kwLStr = ":"
					udFL.itemsDim[0] = 2
				case "ol":
					theDID = getHTMLInfo(S_selection,3)
					if (strlen(theDID)==0)
						udFLSetError(udFL,XMLalert02,-1)
						return -1
					endif
					string/G $theDID = ""
					SVAR theStr = $theDID
					iHTMLOK=0
					do	
						Notebook $nb selection={startOfNextParagraph, EndOfNextParagraph}
						GetSelection notebook, $nb, 3
						theHTML = getHTMLInfo(S_selection,1)
						if (strlen(theHTML)!=0)
							strswitch(theHTML)
								case "li":
									theID = getHTMLInfo(S_selection,3)
									if (strlen(theID)==0)
										udFLSetError(udFL,XMLalert02,-1)
										return -1
									endif
									theScontent = getHTMLInfo(S_selection,0)
									tmpS = theID + kwLStr + theSContent + ";"
									theStr += tmpS
									break
								case "/ol":
									udFL.itemsDim[0] = 1
								case "/ul":
									iHTMLOK = 1
									break
								default:
									break
							endswitch
						endif
					while(!iHTMLOK)
					udFL.itemsType = 2^1
					udFL.itemsCount += 1
					udFL.itemsList += theDID + ";"
					kwLStr = "="
					break
					
				// internal processing of tables to load waves
				
				case "table":
					theDID = getHTMLInfo(S_selection,3)
					if (strlen(theDID)==0)
						udFLSetError(udFL,XMLalert02,-1)
						return -1
					endif
					strswitch(theDID)
						case "row-vector":
							iHTMLOK = 0
							do
								Notebook $nb selection={startOfNextParagraph, EndOfNextParagraph}
								GetSelection notebook, $nb, 3
								theHTML = getHTMLInfo(S_selection,1)
								if (strlen(theHTML)!=0)
									strswitch(theHTML)
										case "tr":
											theID = getHTMLInfo(S_selection,3)
											if (strlen(theID)==0)
												udFLSetError(udFL,XMLalert02,-1)
												return -1
											endif
											break
										case "td":
											theSContent = getHTMLInfo(S_selection,0)
											theSContent = ReplaceString(" ",theSContent,";")
											mxic = ItemsInList(theSContent)
											tmpS = StringFromList(0,theSContent)
											tmpN = str2num(tmpS)
											if (numtype(tmpN)==2)
												udFL.itemsDim[0] = -2
												make/T/O/N=(mxic) $theID
												wave/T theTWave = $theID
												for (ic=0;ic<mxic;ic+=1)
													theTWave[ic] = StringFromList(ic,theSContent)
												endfor
											else
												udFL.itemsDim[0] = -1
												make/O/N=(mxic) $theID
												wave theWave = $theID
												for (ic=0;ic<mxic;ic+=1)
													sscanf StringFromList(ic,theSContent), "%f", tmpN
													theWave[ic] = tmpN
												endfor
											endif
											udFL.itemsType = 2^2
											udFL.itemsCount += 1
											udFL.itemsList += theID + ";"
											break
										case "/table":
											iHTMLOK = 1
											break
										default:
											break
									endswitch
								endif
							while(!iHTMLOK)
							break
						case "waves":
							iHTMLOK = 0
							do
								Notebook $nb selection={startOfNextParagraph, EndOfNextParagraph}
								GetSelection notebook, $nb, 3
								theHTML = getHTMLInfo(S_selection,1)
								if (strlen(theHTML)!=0)
									strswitch(theHTML)
										case "tr":
											theID = getHTMLInfo(S_selection,3)
											if (strlen(theID)==0)
												udFLSetError(udFL,XMLalert02,-1)
												return -1
											endif
											ic = 0
											break
										case "td":
											Notebook $nb findText={"",0}
											GetSelection notebook, $nb, 3
											stPr = V_startParagraph
											Notebook $nb findText={"</td>",0}
											GetSelection notebook, $nb, 3
											enPr = V_startParagraph
											enPo = V_endPos
											Notebook $nb selection={(stPr,0),(enPr,enPo)}
											GetSelection notebook, $nb, 3
											theSContent = getHTMLInfo(S_selection,5)
											if ((exists(theID)!=0) && (ic == 0))
												theID += "_XML"
											endif
											if (ic == 0)
												tmpN = str2num(theSContent)
												if (numtype(tmpN)==2)
													udFL.itemsDim[0] = -2
													make/T/O/N=(1) $theID
													wave/T theTWave = $theID
													theTWave[0] = theSContent
													isNumeric = 0
												else
													udFL.itemsDim[0] = -1
													make/D/O/N=(1) $theID
													wave theWave = $theID
													sscanf theSContent, "%f", tmpN
													theWave[ic] = tmpN
													isNumeric = 1
												endif
												ic = 1
											else
												if (!isNumeric)
													redimension/N=(ic+1) theTWave
													theTWave[ic] = theSContent
												else
													redimension/N=(ic+1) theWave
													sscanf theSContent, "%f", tmpN
													theWave[ic] = tmpN
												endif
												ic += 1
											endif
											break
										case "/table":
											iHTMLOK = 1
											break
										default:
											break
									endswitch
								endif
							while(!iHTMLOK)
							udFL.itemsType = 2^2
							udFL.itemsCount += 1
							udFL.itemsList += theID + ";"
							break
					endswitch
				default:
					break
			endswitch
		else
		
			// start of outer processing for code
			
			theHTML = getHTMLInfo(S_selection,2)
			if (strlen(theHTML)!=0)
				strswitch(theHTML)
					case "code":
						oHTMLOK=0
						do	
							Notebook $nb selection={startOfNextParagraph, EndOfNextParagraph}
							GetSelection notebook, $nb, 3
							theHTML = getHTMLInfo(S_selection,2)
							if (strlen(theHTML)!=0)
								oHTMLOK = 1
							else
								theHTML = getHTMLInfo(S_selection,4)
								if (strlen(theHTML)!=0)
									Execute/Q/Z theHTML
									if (V_flag)
										udFLSetError(udFL,(XMLalert07 + theHTML),0)
										udFLHandleError(udFL)
									endif
								endif
							endif
						while(!oHTMLOK)
						break
					default:
						break
				endswitch
			endif
		endif
		Notebook $nb selection={startOfNextParagraph, EndOfNextParagraph}
		else
			break
		endif
	while (1)
	
	SetDataFolder $cdf
	if(udFL.reportCtrl != 1)
		KillWindow $nb
	endif
	
	return 0

end

// **********************************************
// getHTMLInfo(str, how)
// how: 0 - search for > ... <
//			1 - search for < ... >
//			2 - search for [ ... ]
//			3 - search for id =
//			4 - search for non-blank lines and strip leading spaces or tab characters
//			5 - search for > ... < + strip all returns + all leading/trailing spaces

Static Function/S getHTMLInfo(str, how)
	string str
	variable how
	
	variable pstartLB=0, pstartS=0, pstartRB=0, pstartID=0, tLen, ic
	string theRStr = "", qStr = "\"", eStr = "\\", cStr = "\r", tStr = "\t"
	
	switch(how)
		case 0:
			pstartLB = strSearch(str,">",0)
			pstartRB = strSearch(str,"<",pstartLB)
			theRStr = str[pstartLB+1,pstartRB-1]
			break
		case 1:
			pstartLB = strSearch(str,"<",0)
			if (pstartLB >= 0)
				pstartRB = strsearch(str,">",pstartLB+1)
				pstartS = strsearch(str," ",pstartLB+1)
				if ((pstartS < pstartRB) && (pstartS > pstartLB))
					pstartRB = pstartS
				endif
				theRStr = str[pstartLB+1,pstartRB-1]
			endif
			break
		case 2:
			pstartLB = strSearch(str,"[",0)
			if (pstartLB >= 0)
				pstartRB = strsearch(str,"]",pstartLB+1)
				pstartS = strsearch(str," ",pstartLB+1)
				if ((pstartS < pstartRB) && (pstartS > pstartLB))
					pstartRB = pstartS
				endif
				theRStr = str[pstartLB+1,pstartRB-1]
			endif
			break		
		case 3:
			pstartID = strSearch(str,"id",0)
			if (pstartID>0)
				pstartLB = strsearch(str,qStr,pstartID)
				pstartRB = strsearch(str,qStr,pstartLB+1)
				pstartS = strsearch(str,eStr,pstartLB+1)
				if ((pstartS < pstartRB) && (pstartS > pstartLB))
					pstartRB+=-1
				endif
				theRStr = str[pstartLB+1,pstartRB-1]
			endif
			break
		case 4:
			pstartLB = strSearch(str," ",0)
			if (pstartLB == 0)
				do
					pstartRB = strSearch(str," ",pstartLB)
					if (pstartRB > 0)
						pstartLB +=1
					else
						break
					endif
				while(1)
			else
				pstartLB = strSearch(str,"\t",0)
				if (pstartLB == 0)
					pstartLB = 1
				else
					pstartLB = 0
				endif
			endif
			pstartRB = strlen(str)
			theRStr = str[pstartLB,pstartRB-1]
			break
		case 5:
			pstartLB = strSearch(str,">",0)
			pstartRB = strSearch(str,"<",pstartLB)
			theRStr = ReplaceString(cStr,str[pstartLB+1,pstartRB-1],"")
			theRStr = ReplaceString(tStr,theRStr,"")
			tLen = strlen(theRstr)
			pstartLB = 0
			for (ic=0;ic<tLen;ic+=1)
				if (cmpstr(theRStr[ic]," ")!=0)
					break
				endif
			endfor
			pstartLB = ic
			for (ic=tLen-1;ic>0;ic+=-1)
				if (cmpstr(theRStr[ic]," ")!=0)
					break
				endif
			endfor
			pstartRB = ic
			theRStr = theRStr[pstartLB,pstartRB]
			break
	endswitch
	
	return theRStr
	
end


// **********************************************
// parseXMLStr(lstr,xhow)
// lstr: string (line) containing value to parse
// xhow:	0 - replace carriage returns properly

Static Function/S parseXMLStr(lstr,xhow)
	variable xhow
	string lstr
	
	string rstr, crs = "\r"
	variable so, sf
	
	switch(xhow)
		case 0:
			rstr = ReplaceString("\\\\r",lstr,crs)
			break
	endswitch
	
	return rstr
end
